# How to Run the Online Reconstruction

## Scheduler (Optional)
Start a luigi daemon (preferably in a screen session) on a certain `HOST` and using a certain `PORT`
```shell
luigid --port PORT
```

## Actual Reconstruction
```shell
python3 <path/to>/job_handler.py --batch --scheduler-host HOST --scheduler-port PORT
```

# New Machine Names after KEKcc Upgrade:

- `ccw01` &rarr; `kekts`
- `ccw02` &rarr; `kekt`
- `ccw03` &rarr; `kekrs`, `kekas`, `kekms`
- `ccw04` &rarr; `kekr`
- `ccw05` &rarr; `keka`
- `ccw06` &rarr; `None` (Some Errors occur on this machine!)
- `ccw07` &rarr; `kekm`
- `ccx01` &rarr; `kekfs`
- `ccx02` &rarr; `kekf`
- `ccx03` &rarr; `kekps`
- `ccx04` &rarr; `kekp`


# Work Share for Production of Sep 2020 (`08thSep2020`)

| Stream  | Setup | Status | Hint   |
| :------ | :---- | :---:  | :---:  |
| 0 Data  | F     | ---    | **Pull and Compile!** |
| 0 Gen   | F     | ---    | **Pull and Compile!** |
| 0 Gap   | F     | ---    | **Pull and Compile!** |
| 0 `D**` | F     | ---    | **Pull and Compile!** |
| 0 Rest  | F     | ---    | **Pull and Compile!** |
| 1       | P     | ---    | **Pull and Compile!** |
| 2       | R     | ---    | **Pull and Compile!** |
| 3       | T     | ---    | **Pull and Compile!** |
| 4       | A     | ---    | **Switch to master, Pull and Compile!** |
| 5       | M     | ---    | **Pull and Compile!** |
| 6       | P     | ---    | **Pull and Compile!** |
| 7       | R     | ---    | **Pull and Compile!** |
| 8       | T     | ---    | **Pull and Compile!** |
| 9       | A     | ---    | **Switch to master, Pull and Compile!** |

# Production Setups:

## Setup F
**Setup in** `/home/belle2/metzner/`

**Data written to** `/gpfs/fs02/belle/users/metzner/rdstar_data/`

**Scheduler on `ccx01`**
```shell
luigid --port 8047
```

**Jobs on `ccx02`**
```shell
python3 /home/belle2/metzner/RDStar/rdstar/batch_jobs/job_handler.py --batch --scheduler-host ccx01 --scheduler-port 8047
```

## Setup P
**Setup in** `/home/belle2/pecker/felix/`

**Data written to** `/gpfs/fs02/belle/users/pecker/felix/rdstar_data/`

**Scheduler on `ccx03`**
```shell
luigid --port 8843
```

**Jobs on `ccx04`**
```shell
python3 /home/belle2/pecker/felix/RDStar/rdstar/batch_jobs/fp_job_handler.py --batch --scheduler-host ccx03 --scheduler-port 8843
```

## Setup T
**Setup in** `/home/belle2/tboeckh/felix/`

**Data written to** `/gpfs/fs02/belle/users/tboeckh/felix/rdstar_data/`

**Scheduler on `ccw01`**
```shell
luigid --port 8848
```

**Jobs on `ccw02`**
```shell
python3 /home/belle2/tboeckh/felix/RDStar/rdstar/batch_jobs/tobias_job_handler.py --batch --scheduler-host ccw01 --scheduler-port 8848
```

## Setup R
**Setup in** `/home/belle2/rvantond/felix/`

**Data written to** `/gpfs/fs02/belle/users/rvantond/felix/rdstar_data/`

**Scheduler on `ccw03`**
```shell
luigid --port 8849
```

**Jobs on `ccw04`**
```shell
python3 /home/belle2/rvantond/felix/RDStar/rdstar/batch_jobs/ray_job_handler.py --batch --scheduler-host ccw03 --scheduler-port 8849
```

## Setup A
**Setup in** `/home/belle2/abaur/felix/`

**Data written to** `/gpfs/fs02/belle/users/abaur/felix/rdstar_data/`

**Scheduler on `ccw03`**
```shell
luigid --port 8842
```

**Jobs on `ccw05`**
```shell
python3 /home/belle2/abaur/felix/RDStar/rdstar/batch_jobs/abaur_job_handler.py --batch --scheduler-host ccw03 --scheduler-port 8842
```

## Setup M
**Setup in** `/gpfs/fs02/belle2/users/mapr/felix/` = `/home/belle2/mapr/felix/`

**Data written to** `/gpfs/fs02/belle2/users/mapr/felix_rdstar_data/rdstar_data/`

**Scheduler on `ccw03`**
```shell
luigid --port 8841
```

**Jobs on `ccw07`**
```shell
python3 /home/belle2/mapr/felix/RDStar/rdstar/batch_jobs/mapr_job_handler.py --batch --scheduler-host ccw03 --scheduler-port 8841
```

# Data to be Processed

| Data Type          | Category   | Streams  | Purpose        | Covered in Configs |
| :----------------- | :--------- | :------- | :------------- | :----------------  |
| Data Y(4S)         | Data       | 0 (None) | Data           | Yes                |
| Data Off Resonance | Data       | 0 (None) | Continuum Data | Yes                |
| Charged            | Generic    | 0 - 9    | MC             | Yes                |
| Mixed              | Generic    | 0 - 9    | MC             | Yes                |
| Charm              | Generic    | 0 - 5    | MC             | Yes                |
| UDS                | Generic    | 0 - 5    | MC             | Yes                |
| Rare               | Rare MC    | = 50     | MC (rare)      | Yes                |
| ulnu               | Rare MC    | = 20     | MC (rare)      | Yes                |
| JPsi               | Special MC | = 100    | MC (special)   | Yes                |
| Psi2S              | Special MC | = 100    | MC (special)   | Yes                |
| `D**`              | Signal MC  | 0 (None) | MC             | Yes                |
| Gap                | Signal MC  | 0 (None) | MC             | Yes                |
| Gap Via `D**`      | Signal MC  | 0 (None) | MC             | Yes                |
| Signal `D` Eff     | Signal MC  | 0 (None) | Eff            | Yes                |
| Signal `D` MVA     | Signal MC  | 0 (None) | MVA            | Yes                |
| Signal `D*` EFF    | Signal MC  | 0 (None) | Eff            | Yes                |
| Signal `D*` MVA    | Signal MC  | 0 (None) | MVA            | Yes                |
| `D**` (pi0) Eff    | Signal MC  | 0 (None) | Eff            | Yes                |
| `D**` (pi0) MVA    | Signal MC  | 0 (None) | MVA            | Yes                |
| `D**` (pi+) Eff    | Signal MC  | 0 (None) | Eff            | Yes                |
| `D**` (pi+) MVA    | Signal MC  | 0 (None) | MVA            | Yes                |

### Private Monte Carlo Subsets

* **`D**`**
    -
    - BZD1E
    - BZD0sE
    - BZDp1E
    - BZD2sE
    - BZD1Mu
    - BZD0sMu
    - BZDp1Mu
    - BZD2sMu
    - BMD1E
    - BMD0sE
    - BMDp1E
    - BMD2sE
    - BMD1Mu
    - BMD0sMu
    - BMDp1Mu
    - BMD2sMu
* **Gap**
    -
    - **GapD1Mixed**
    - GapDEtaMixed
    - GapDstEtaMixed
    - GapDPiPiMixed
    - GapDstPiPiMixed
    - **GapD1Charged**
    - GapDEtaCharged
    - GapDstEtaCharged
    - GapDPiPiCharged
    - GapDstPiPiCharged
* **Gap Via `D**`**
    -
    - GapDEtaViaD0sMixed
    - GapDstEtaViaDp1Mixed
    - GapDPiPiViaDp1Mixed
    - GapDPiPiViaD0sMixed
    - GapDstPiPiViaDp1Mixed
    - GapDstPiPiViaD0sMixed
    - GapDEtaViaD0sCharged
    - GapDstEtaViaDp1Charged
    - GapDPiPiViaDp1Charged
    - GapDPiPiViaD0sCharged
    - GapDstPiPiViaDp1Charged
    - GapDstPiPiViaD0sCharged
* **Signal `D` Efficiency**
    -
    - DME
    - DMMu
    - DMTau
    - DZE
    - DZMu
    - DZTau
* **Signal `D*` Efficiency**
    -
    - DSME
    - DSMMu
    - DSMTau
    - DSZE
    - DSZMu
    - DSZTau
* **Signal `D` MVA**
    -
    - DMEMVA
    - DMMuMVA
    - DMTauMVA
    - DZEMVA
    - DZMuMVA
    - DZTauMVA
* **Signal `D*` MVA**
    -
    - DSMEMVA
    - DSMMuMVA
    - DSMTauMVA
    - DSZEMVA
    - DSZMuMVA
    - DSZTauMVA
* **`D**` (pi0) Efficiency**
    -
    - BZD1EEff
    - BZD0sEEff
    - BZDp1EEff
    - BZD2sEEff
    - BZD1MuEff
    - BZD0sMuEff
    - BZDp1MuEff
    - BZD2sMuEff
    - BMD1EEff
    - BMD0sEEff
    - BMDp1EEff
    - BMD2sEEff
    - BMD1MuEff
    - BMD0sMuEff
    - BMDp1MuEff
    - BMD2sMuEff
* **`D**` (pi0) MVA**
    -
    - BZD1EMVA
    - BZD0sEMVA
    - BZDp1EMVA
    - BZD2sEMVA
    - BZD1MuMVA
    - BZD0sMuMVA
    - BZDp1MuMVA
    - BZD2sMuMVA
    - BMD1EMVA
    - BMD0sEMVA
    - BMDp1EMVA
    - BMD2sEMVA
    - BMD1MuMVA
    - BMD0sMuMVA
    - BMDp1MuMVA
    - BMD2sMuMVA
* **`D**` (pi+) Efficiency**
    -
    - BZD1EChargedPiEff
    - BZDp1EChargedPiEff
    - BZD2sEChargedPiEff
    - BZD0sEChargedPiEff
    - BZD1MuChargedPiEff
    - BZDp1MuChargedPiEff
    - BZD2sMuChargedPiEff
    - BZD0sMuChargedPiEff
    - BMD1EChargedPiEff
    - BMDp1EChargedPiEff
    - BMD2sEChargedPiEff
    - BMD0sEChargedPiEff
    - BMD1MuChargedPiEff
    - BMDp1MuChargedPiEff
    - BMD2sMuChargedPiEff
    - BMD0sMuChargedPiEff
* **`D**` (pi+) MVA**
    -
    - BZD1EChargedPiMVA
    - BZDp1EChargedPiMVA
    - BZD2sEChargedPiMVA
    - BZD0sEChargedPiMVA
    - BZD1MuChargedPiMVA
    - BZDp1MuChargedPiMVA
    - BZD2sMuChargedPiMVA
    - BZD0sMuChargedPiMVA
    - BMD1EChargedPiMVA
    - BMDp1EChargedPiMVA
    - BMD2sEChargedPiMVA
    - BMD0sEChargedPiMVA
    - BMD1MuChargedPiMVA
    - BMDp1MuChargedPiMVA
    - BMD2sMuChargedPiMVA
    - BMD0sMuChargedPiMVA


## Helpful Commandline Tools for Config Modifications:

**Modify String:**

`find ./rdstar/configs/prod_08thSep2020_stream?/*.json -type f -exec sed -i 's/old_string/new_string/g' {} \;`

**Delete a line:**

`find ./rdstar/configs/prod_08thSep2020_stream?/*.json -type f -exec sed -i '/patter_to_match_line/d' {} \;`