# Belle MC production for the R(D^(*)) analysis

This package contains a python wrapper and decay files for private signal MC production
using the `basf` software on the `kekcc`.

## Usage of the Belle Software `basf` after upgrade of KEKcc on 1st of September 2020

Infos are provided by [Nishida-san on this overview page](https://wiki.kek.jp/display/~nishida6/Information+on+KEKCC+2020+for+Belle+users).

The important infos are:

* Sample `.bashrc` or `.cshrc` for basf environment can be found by `~nishida6/public/Env/bashrc` or `~nishida6/public/Env/cshrc` on KEKcc.
* Belle library is installed under `/sw/belle` on KEKcc.
* [Belle File search](http://bweb3.cc.kek.jp/) can be accessed only inside KEK. Use VPN to access it from outside. You may also want to use the `check_process_url` command.

Issues with `basf` on `CentOS7` have been reported and fixed now.
Details are documented on
[the dedicated website mentioned above](https://wiki.kek.jp/display/~nishida6/Information+on+KEKCC+2020+for+Belle+users).


## General Infos
We can write do a dedicated directory
```bash
/group/belle2/TMP/metzner
```
provided to us by Takanori Hara.
It has a limit of `35 TB` which cannot be extended... except for when Hara-san extends it to 60 TB.

The accounts `metzner`, `pecker`, `rvantond` and `tboeckh` can write to this directory.

If the data is not used frequently anymore, it should be moved to the HSM tape system.


## Event Generation
 1. Make sure you are running `bash` (required for `basf`) and all required scripts are executable,
 including `runRvtgen` and the `.csh` files in the `evtgen` directory.
 2. Setup the `basf` environment by executing the following command
     ```bash
     source ./setup_basf.sh
     ```
 3. Run the script with
    ```bash
    ./mc_generation.py -t DECAY_TYPE -j MAX_NUMBER_OF_EVTS_PER_JOB -o PATH_TO_OUTPUT_DIRECTORY [--test] &> <PATH_TO_OUTPUT_DIRECTORY>/mc_gen_<DECAY_TYPE>_DDthMONTHYYYY.log
    ```
    where `MAX_NUMBER_OF_EVTS_PER_JOB` is the number of events that will be produced in each batch job.

    `PATH_TO_OUTPUT_DIRECTORY` should be `/group/belle2/TMP/metzner/<some_production_identifier>/`
    so that the decay key is part of the path.

    You can run the script with `./mc_generation.py -h` to get an overview of the allowed command line arguments.
    Available decay types are:
    * `Dlnu`: B -> D l nu decays (for efficiency calculations)
    * `Dstlnu`: B -> D* l nu decays (for efficiency calculations)
    * `DlnuMVA`: B -> D l nu decays (for MVA trainings; same dec files as `Dlnu`)
    * `DstlnuMVA`: B -> D* l nu decays (for MVA trainings; same dec files as `DstlnuMVA`)
    * `Dststlnu`: B -> D** l nu decays **(Replacing generic samples)**
    * `SignalDststlnupiz`: B -> D** (-> D(*) pi^0) l nu decays (for efficiency calculations)
    * `SignalDststlnupim`: B -> D** (-> D(*) pi^-) l nu decays (for efficiency calculations)
    * `SignalDststlnupizMVA`: B -> D** (-> D(*) pi^0) l nu decays (for MVA trainings; same dec files as `SignalDststlnupiz`)
    * `SignalDststlnupimMVA`: B -> D** (-> D(*) pi^-) l nu decays (for MVA trainings; same dec files as `SignalDststlnupim`)
    * `Dststtaunu`: B -> D** tau nu decays **(Optional; Replacing generic samples)**
    * `SignalDststtaunupiz`: B -> D** (-> D(*) pi^0) tau nu decays (**Optional**; for efficiency calculations)
    * `SignalDststtaunupim`: B -> D** (-> D(*) pi^-) tau nu decays (**Optional**; for efficiency calculations)
    * `GapD1`: B -> Gap B -> D1 (-> D pi pi) l nu decays **(Complementing generic samples)**
    * `OldGapDEta`: B -> Gap B -> D(*) eta l nu decays **(Complementing generic samples) (Old Version w\o resonance)**
    * `OldGapDPiPi`: B -> Gap B -> D(*) pi pi l nu decays **(Complementing generic samples) (Old Version w\o resonance)**
    * `GapDEtaViaDstst`: B -> Gap B -> `D**` (-> D(*) eta) l nu decays **(Complementing generic samples) (with `D**` = `D'_1`, `D_0*`)**
    * `GapDPiPiViaDstst`: B -> Gap B -> `D**` (-> D(*) pi pi) l nu decays **(Complementing generic samples) (with `D**` = `D'_1`, `D_0*`)**

    The total number of events produced per decay file for the given decay type is fixed
    and specified in the `n_events_dict` dictionary in `mc_generation.py`.

    With the optional flag `--test` the setup can be tested.
    The script will produce 1000 events of the given decay type in 10 jobs.

You can find the `.dec` files of these events in the `dec_files/` directory.
If you want to add new `.dec` files, create a directory in `dec_files/`
and add the decay types to the `dec_file_dirs` in `mc_generation.py` and `mc_simulation.py`.

After the `evtgen` jobs ran through one can tidy up with the command

```bash
./mc_generation.py -t DECAY_TYPE -n TOTAL_NUMBER_OF_EVTS -j MAX_NUMBER_OF_EVTS_PER_JOB -o PATH_TO_OUTPUT_DIRECTORY --clean
```

This command will remove the various job scripts, which were
created during the MC generation, as well as the copied decay files.

### Event Generation Summary
You can check if the right number of events was produced by `evtgen` by running the `makeSummaryEvtgen` script in
the `evtgen` directory:
```bash
./evtgen/makeSummaryEvtgen -MC <PATH_TO_OUTPUT_DIRECTORY>/<DECAY_TYPE>/evtgen/log/*/evtgen* &> <PATH_TO_OUTPUT_DIRECTORY>/mc_gen_summary_<DECAY_TYPE>_DDthMONTHYYYY.log
```
where `PATH_TO_OUTPUT_DIRECTORY` is the same directory as used in the call of the `mc_generation.py` script.
The sum of events produced should correspond to the total number of events you requested. 
    
## Event Simulation
 1. Again all relevant scripts must be executable: `mc_simulation.py` and the `.csh` scripts in the `gsim` directory.
 2. Run the `mc_simulation.py` script with the following parameters:
    * `-t`: Decay type, e.g. `-t Dlnu`
    * `-i`: Input directory, e.g. `-i PATH_TO_OUTPUT_DIRECTORY`,
    where `PATH_TO_OUTPUT_DIRECTORY` is the path to the directory specified for the event generation
    * `[-s]`: Optional argument `sub_dec_type` to specify a `SUB_DECAY_TYPE`,
    meaning one of the decay files comprising the decay type.
    E.g. `DoneminusE` for the decay type `Dststlnu`.

    
```bash
./mc_simulation.py -t DECAY_TYPE -i PATH_TO_OUTPUT_DIRECTORY [-s SUB_DECAY_TYPE] &> <PATH_TO_OUTPUT_DIRECTORY>/mc_sim_<DECAY_TYPE>[_<SUB_DECAY_TYPE>]_DDthMONTHYYYY.log
```
    
The `mdst` files will be placed in `<PATH_TO_OUTPUT_DIRECTORY>/<DECAY_TYPE>/gsim/mdst/`.

Also here, you can tidy up by running `mc_simulation.py -t x -i x --clean`, which will
remove all job scripts in the `gsim` directory. Again, this must be done manually after
all gsim jobs completed successfully.

### Event Simulation Summary
To get the number of actually produced events, you should run the script `makeSummaryGsim`
which can be found in the `gsim` directory:
```bash
./gsim/makeSummaryGsim -MC <PATH_TO_OUTPUT_DIRECTORY>/<DECAY_TYPE>/gsim/log/*/evtgen_exp_*-*[!gr].log  &> <PATH_TO_OUTPUT_DIRECTORY>/mc_sim_summary_<DECAY_TYPE>_DDthMONTHYYYY.log
```
Here `log/*/evtgen_exp_*-*[!gr].log` should parse **all** `.log` files and omit the `generror.log` and `genlog.log` files.

This report will generate a count of the number of events generated. 
You can also use `log/<decay_type>/evtgen_exp_*-?.log` to get the number of events produced for the specified decay type.

This check will also show you which jobs failed. You can rerun them by looking at the respective log files
and rerun them via the following command executed in the `gsim` directory:
```bash
bsub -q l -e <PATH_TO_OUTPUT_DIRECTORY>/gsim/log/<data_type>/evtgen_exp_<...>error.log -o <PATH_TO_OUTPUT_DIRECTORY>/gsim/log/<data_type>/evtgen_exp_<...>log.log ./rec.csh <PATH_TO_OUTPUT_DIRECTORY>/evtgen/gen/<data_type>/evtgen_exp_<...>.gen <exp_num> <data_type> <PATH_TO_OUTPUT_DIRECTORY>
```
where some job specific strings have to be adapted.
These can be extracted from the respective log files.

You can resubmit failed `GSim` jobs using the `resubmit_gsim.py` script (in the `gsim` folder).
For that pipe the output from `./makeSummaryGsim -MC <PATH_TO_OUTPUT_DIRECTORY>gsim/log/*/evtgen_exp_*-?.log` into a file.
Failed jobs are then automatically resubmitted with
```bash
./resubmit_gsim -i <SUMMARY_FILE.txt>
```


## Status --- Production of August 2020

| Data Type              | User | EvtGen       | EvtGen Check | GSim         | GSim Check   | Size     | Redo Mixed |
| :--------------------- | :--- | :----------- | -----------: | :----------- | -----------: | :------- | :--------- |
| `Dlnu`                 | F    | Done         | 15000000     | Done         | 14983309     | 314 GB   |            |
| `Dstlnu`               | P    | Done         | 15000000     | Done         | 14983034     | 330 GB   |            |
| `DlnuMVA`              | R    | Done         | 15000000     | Done         | 14983188     | 314 GB   |            |
| `DstlnuMVA`            | T    | Done         | 15000000     | Done         | 14983113     | 330 GB   |            |
| `Dststlnu` (see below) | F    | Done         | 1167999808   | Done         | 1167214317   | 26 TB    |            |
| `SignalDststlnupiz`    | P    | Done         | 40000000     | Done         | 39974543     | 884 GB   |            |
| `SignalDststlnupim`    | R    | Done         | 40000000     | Done         | 39976210     | 910 GB   |            |
| `SignalDststlnupizMVA` | T    | Done         | 40000000     | Done         | 39974552     | 885 GB   |            |
| `SignalDststlnupimMVA` | P    | Done         | 40000000     | Done         | 39976028     | 910 GB   |            |
| `Dststtaunu`           |      |              |              |              |              |          |            |
| `SignalDststtaunupiz`  |      |              |              |              |              |          |            |
| `SignalDststtaunupim`  |      |              |              |              |              |          |            |
| `GapD1`                | R    | Done         | 99999998     | Done         | 99919164     | 2.3 TB   |            |
| `GapDEta`              | T    | Done         | 159999976    | Done         | 159931946    | 3.7 TB   |            |
| `GapDPiPi`             | P    | Done         | 18999994     | Done         | 18994754     | 440 GB   |            |
| `GapDstPiPi`           | R    | Done         | 91999988     | Done         | 91975653     | 2.2 TB   |            |
| `GapDEtaViaDstst`      | T    | Done         | 159999976    | Done         | 159907414    | 3.7 TB   |            |
| `GapDPiPiViaDstst`     | P    | Done         | 23999988     | Done         | 23981226     | 559 GB   |            |
| `GapDstPiPiViaDstst`   | R    | Done         | 100000008    | Done         | 99917531     | 2.4 TB   |            |

### Status of `Dststlnu` Subsets

| `Dststlnu` Sub Type   | EvtGen Check | User | GSim         | GSim Check   | Size     | Redo Mixed |
| :-------------------- | -----------: | :--- | :----------- | -----------: | :------- | :--------: |
| `DoneminusE`          | 72999988     | F    | Done         | 72969720     | 1.6 TB   |            |
| `DoneminusMu`         | 72999988     | F    | Done         | 72934855     | 1.6 TB   |            |
| `DonezeroE`           | 72999988     | F    | Done         | 72972061     | 1.6 TB   | ---        |
| `DonezeroMu`          | 72999988     | F    | Done         | 72939807     | 1.6 TB   | ---        |
| `DprimeoneminusE`     | 72999988     | F    | Done         | 72969660     | 1.6 TB   |            |
| `DprimeoneminusMu`    | 72999988     | F    | Done         | 72932851     | 1.6 TB   |            |
| `DprimeonezeroE`      | 72999988     | F    | Done         | 72965689     | 1.6 TB   | ---        |
| `DprimeonezeroMu`     | 72999988     | P    | Done         | 72889052     | 1.6 TB   | ---        |
| `DtwostarminusE`      | 72999988     | P    | Done         | 72973031     | 1.6 TB   |            |
| `DtwostarminusMu`     | 72999988     | P    | Done         | 72943431     | 1.6 TB   |            |
| `DtwostarzeroE`       | 72999988     | P    | Done         | 72975847     | 1.5 TB   | ---        |
| `DtwostarzeroMu`      | 72999988     | P    | Done         | 72949175     | 1.5 TB   | ---        |
| `DzerostarminusE`     | 72999988     | R    | Done         | 72949473     | 1.5 TB   |            |
| `DzerostarminusMu`    | 72999988     | T    | Done         | 72939202     | 1.5 TB   |            |
| `DzerostarzeroE`      | 72999988     | R    | Done         | 72970858     | 1.5 TB   | ---        |
| `DzerostarzeroMu`     | 72999988     | T    | Done         | 72939605     | 1.5 TB   | ---        |


# Reproduction of Mixed Samples

Due to a misconfiguration of all decay files of mixed B meson decays, the respective samples have to be redone.

| Data Type                    | User | Del Old | EvtGen       | EvtGen Check | GSim         | GSim Check   |
| :---------------------       | :--- | :------ | :----------- | -----------: | :----------- | -----------: |
| `Dlnu`                       | F    | Done    | Done         | 15000000     | Done         | 14983737     |
| `Dstlnu`                     | P    | Done    | Done         | 15000000     | Done         | 14983943     |
| `DlnuMVA`                    | R    | Done    | Done         | 15000000     | Done         | 14983755     |
| `DstlnuMVA`                  | T    | Done    | Done         | 15000000     | Done         | 14984083     |
| **`D**`** `DoneminusE`       | F    | Done    | Done         | 72999988     | Done         | 72968227     |
| **`D**`** `DoneminusMu`      | F    | Done    | Done         | 72999988     | Running      |              |
| **`D**`** `DprimeoneminusE`  | P    | Done    | Done         | 72999988     | Done         | 72969667     |
| **`D**`** `DprimeoneminusMu` | P    | Done    | Done         | 72999988     | Running      |              |
| **`D**`** `DtwostarminusE`   | R    | Done    | Done         | 72999988     | Done         | 72973227     |
| **`D**`** `DtwostarminusMu`  | R    | Done    | Done         | 72999988     | Running      |              |
| **`D**`** `DzerostarminusE`  | T    | Done    | Done         | 72999988     | Done         | 72970500     |
| **`D**`** `DzerostarminusMu` | T    | Done    | Done         | 72999988     | Running      |              |
| `SignalDststlnupiz`          | F    | Done    | Done         | 40000000     | Done         | 39974978     |
| `SignalDststlnupim`          | P    | Done    | Done         | 40000000     | Done         | 39976102     |
| `SignalDststlnupizMVA`       | R    | Done    | Done         | 40000000     | Done         | 39975142     |
| `SignalDststlnupimMVA`       | T    | Done    | Done         | 40000000     | Done         | 39976143     |
| `GapD1`                      | F    | Done    | Done         | 99999998     | Done         | 99920000     |
| `GapDEta`                    | P    | Done    | Done         | 159999976    | Done         | 159931977    |
| `GapDPiPi`                   | R    | Done    | Done         | 18999994     | Done         | 18994712     |
| `GapDstPiPi`                 | T    | Done    | Done         | 91999988     | Done         | 91975871     |
| `GapDEtaViaDstst`            | R    | Done    | Done         | 159999976    | Checking     |              |
| `GapDPiPiViaDstst`           | F    | Done    | Done         | 23999988     | Done         | 23981522     |
| `GapDstPiPiViaDstst`         | T    | Done    | Done         | 100000008    | Done         | 99919468     |


## Problematic Productions:

### `Dststlnu` --- `DoneminusE` (F) **RESUBMITTED**
Comparing
`/group/belle2/TMP/metzner/mc_production_august_2020/problem_check_mc_new_mixed_gen_summary_Dststlnu_DoneminusE_15thSep2020.log`
and
`/group/belle2/TMP/metzner/mc_production_august_2020/problem_check_mc_new_mixed_sim_summary_Dststlnu_DoneminusE_15thSep2020.log`

```python
(72999988 - 72306378) - 10 * 50000 - (250000 - 21156 - 25568 - 5152 - 23 - 4491)
```

Warnings for
- `DoneminusE/evtgen_exp_25_DoneminusE-1.log` 50000
- `DoneminusE/evtgen_exp_27_DoneminusE-47.log` 50000
- `DoneminusE/evtgen_exp_31_DoneminusE-27.log` 50000
- `DoneminusE/evtgen_exp_33_DoneminusE-19.log` 50000
- `DoneminusE/evtgen_exp_37a_DoneminusE-25.log` 50000
- `DoneminusE/evtgen_exp_37b_DoneminusE-30.log` 50000
- `DoneminusE/evtgen_exp_37b_DoneminusE-49.log` 50000
- `DoneminusE/evtgen_exp_45a_DoneminusE-0.log` 50000
- `DoneminusE/evtgen_exp_55_DoneminusE-109.log` 50000
- `DoneminusE/evtgen_exp_63_DoneminusE-52.log` 50000

**The Problematic Ones:**
```bash
gen: DoneminusE/evtgen_exp_61_DoneminusE-2.log  | 50000
sim: DoneminusE/evtgen_exp_61_DoneminusE-2.log  | 21156
```

```bash
gen: DoneminusE/evtgen_exp_61_DoneminusE-31.log | 50000
sim: DoneminusE/evtgen_exp_61_DoneminusE-31.log | 25568
```

```bash
gen: DoneminusE/evtgen_exp_61_DoneminusE-34.log | 50000
sim: DoneminusE/evtgen_exp_61_DoneminusE-34.log | 5152
```

```bash
gen: DoneminusE/evtgen_exp_61_DoneminusE-36.log | 50000
sim: DoneminusE/evtgen_exp_61_DoneminusE-36.log | 23
```

```bash
gen: DoneminusE/evtgen_exp_61_DoneminusE-9.log  | 50000
sim: DoneminusE/evtgen_exp_61_DoneminusE-9.log  | 4491
```

### `Dststlnu` --- `DtwostarminusE` (R) **RESUBMITTED**
Comparing
`/group/belle2/TMP/metzner/mc_production_august_2020/problem_check_mc_new_mixed_gen_summary_Dststlnu_DtwostarminusE_15thSep2020.log`
and
`/group/belle2/TMP/metzner/mc_production_august_2020/problem_check_mc_new_mixed_sim_summary_Dststlnu_DtwostarminusE_15thSep2020.log`

```python
(72999988 - 71599632) - 24 * 50000 - 48096 - (250000 - 3633 - 27546 - 14217 - 29449 - 22895)
```

Warnings for
- `DtwostarminusE/evtgen_exp_15_DtwostarminusE-5.log` 50000
- `DtwostarminusE/evtgen_exp_19a_DtwostarminusE-0.log` 50000
- `DtwostarminusE/evtgen_exp_33_DtwostarminusE-11.log!` 50000
- `DtwostarminusE/evtgen_exp_35_DtwostarminusE-30.log` 50000
- `DtwostarminusE/evtgen_exp_37a_DtwostarminusE-37.log` 50000
- `DtwostarminusE/evtgen_exp_37a_DtwostarminusE-38.log` 50000
- `DtwostarminusE/evtgen_exp_37a_DtwostarminusE-39.log` 50000
- `DtwostarminusE/evtgen_exp_37a_DtwostarminusE-56.log` 50000
- `DtwostarminusE/evtgen_exp_37a_DtwostarminusE-57.log` 48096
- `DtwostarminusE/evtgen_exp_37a_DtwostarminusE-5.log` 50000
- `DtwostarminusE/evtgen_exp_37a_DtwostarminusE-6.log` 50000
- `DtwostarminusE/evtgen_exp_37a_DtwostarminusE-7.log` 50000
- `DtwostarminusE/evtgen_exp_37a_DtwostarminusE-8.log` 50000
- `DtwostarminusE/evtgen_exp_37a_DtwostarminusE-9.log` 50000
- `DtwostarminusE/evtgen_exp_37b_DtwostarminusE-3.log` 50000
- `DtwostarminusE/evtgen_exp_37b_DtwostarminusE-4.log` 50000
- `DtwostarminusE/evtgen_exp_37b_DtwostarminusE-50.log` 50000
- `DtwostarminusE/evtgen_exp_37b_DtwostarminusE-51.log` 50000
- `DtwostarminusE/evtgen_exp_41a_DtwostarminusE-46.log` 50000
- `DtwostarminusE/evtgen_exp_41b_DtwostarminusE-13.log` 50000
- `DtwostarminusE/evtgen_exp_43_DtwostarminusE-17.log` 50000
- `DtwostarminusE/evtgen_exp_43_DtwostarminusE-18.log` 50000
- `DtwostarminusE/evtgen_exp_49_DtwostarminusE-28.log` 50000
- `DtwostarminusE/evtgen_exp_51_DtwostarminusE-60.log` 50000
- `DtwostarminusE/evtgen_exp_55_DtwostarminusE-97.log` 50000

**The Problematic Ones:**
```bash
gen: DtwostarminusE/evtgen_exp_61_DtwostarminusE-0.log  | 50000
sim: DtwostarminusE/evtgen_exp_61_DtwostarminusE-0.log  | 3633
```

```bash
gen: DtwostarminusE/evtgen_exp_61_DtwostarminusE-51.log | 50000
sim: DtwostarminusE/evtgen_exp_61_DtwostarminusE-51.log | 27546
```

```bash
gen: DtwostarminusE/evtgen_exp_61_DtwostarminusE-52.log | 50000
sim: DtwostarminusE/evtgen_exp_61_DtwostarminusE-52.log | 14217
```

```bash
gen: DtwostarminusE/evtgen_exp_61_DtwostarminusE-53.log | 50000
sim: DtwostarminusE/evtgen_exp_61_DtwostarminusE-53.log | 29449
```

```bash
gen: DtwostarminusE/evtgen_exp_61_DtwostarminusE-54.log | 50000
sim: DtwostarminusE/evtgen_exp_61_DtwostarminusE-54.log | 22895
```

### `GapDEta` (P) **RESUBMITTED**
Comparing
`/gpfs/group/belle2/TMP/metzner/mc_production_august_2020/problem_check_mc_new_mixed_gen_summary_GapDEta_15thSep2020.log`
and
`/gpfs/group/belle2/TMP/metzner/mc_production_august_2020/problem_check_mc_new_mixed_sim_summary_GapDEta_15thSep2020.log`
```python
(159999976 - 159861635) - 50000 - (50000 - 6121) - (50000 - 5538)
```

Warnings for
- `GapDetaMixed/evtgen_exp_61_GapDetaMixed-37.log` 50000

**The Problematic Ones:**
```bash
gen: /group/belle2/TMP/metzner/mc_production_august_2020/GapDEta/log/GapDetaMixed/evtgen_exp_61_GapDetaMixed-13.log  | 50000
sim: /group/belle2/TMP/metzner/mc_production_august_2020/GapDEta/log/GapDetaMixed/evtgen_exp_61_GapDetaMixed-13.log  | 5538
```

```bash
gen: /group/belle2/TMP/metzner/mc_production_august_2020/GapDEta/log/GapDetaMixed/evtgen_exp_61_GapDetaMixed-27.log  | 50000
sim: /group/belle2/TMP/metzner/mc_production_august_2020/GapDEta/log/GapDetaMixed/evtgen_exp_61_GapDetaMixed-27.log  | 6121
```

### `GapDstPiPi` (T) **RESUBMITTED**
Comparing
`/group/belle2/TMP/metzner/mc_production_august_2020/problem_check_mc_new_mixed_gen_summary_GapDstPiPi_15thSep2020.log`
and
`/group/belle2/TMP/metzner/mc_production_august_2020/problem_check_mc_new_mixed_sim_summary_GapDstPiPi_15thSep2020.log`

```python
(91999988 - 91279976) -13 * 50000 - (50000 - 22732) - (50000 - 7256)
```

Warnings for
- `GapDstPiPi/log/GapDstpipiMixed/evtgen_exp_35_GapDstpipiMixed-10.log` 50000
- `GapDstPiPi/log/GapDstpipiMixed/evtgen_exp_37a_GapDstpipiMixed-25.log` 50000
- `GapDstPiPi/log/GapDstpipiMixed/evtgen_exp_37a_GapDstpipiMixed-28.log` 50000
- `GapDstPiPi/log/GapDstpipiMixed/evtgen_exp_37b_GapDstpipiMixed-25.log` 50000
- `GapDstPiPi/log/GapDstpipiMixed/evtgen_exp_37b_GapDstpipiMixed-27.log` 50000
- `GapDstPiPi/log/GapDstpipiMixed/evtgen_exp_37b_GapDstpipiMixed-28.log` 50000
- `GapDstPiPi/log/GapDstpipiMixed/evtgen_exp_39_GapDstpipiMixed-16.log` 50000
- `GapDstPiPi/log/GapDstpipiMixed/evtgen_exp_39_GapDstpipiMixed-45.log` 50000
- `GapDstPiPi/log/GapDstpipiMixed/evtgen_exp_39_GapDstpipiMixed-9.log` 50000
- `GapDstPiPi/log/GapDstpipiMixed/evtgen_exp_47_GapDstpipiMixed-42.log` 50000 **`Runtime error`!**
- `GapDstPiPi/log/GapDstpipiMixed/evtgen_exp_47_GapDstpipiMixed-4.log` 50000
- `GapDstPiPi/log/GapDstpipiMixed/evtgen_exp_49_GapDstpipiMixed-6.log` 50000
- `GapDstPiPi/log/GapDstpipiMixed/evtgen_exp_51_GapDstpipiMixed-47.log` 50000

**The Problematic Ones:**
```bash
gen: GapDstpipiMixed/evtgen_exp_61_GapDstpipiMixed-16.log
sim: GapDstpipiMixed/evtgen_exp_61_GapDstpipiMixed-16.log
```

```bash
gen: GapDstpipiMixed/evtgen_exp_61_GapDstpipiMixed-20.log
sim: GapDstpipiMixed/evtgen_exp_61_GapDstpipiMixed-20.log
```

### `GapDEtaViaDstst` (R) **RESUBMITTED**
Comparing
`/gpfs/group/belle2/TMP/metzner/mc_production_august_2020/problem_check_mc_new_mixed_gen_summary_GapDEtaViaDstst_15thSep2020.log`
and
`/gpfs/group/belle2/TMP/metzner/mc_production_august_2020/problem_check_mc_new_mixed_sim_summary_GapDEtaViaDstst_15thSep2020.log`

```python
(159999976 - 159056258) - 18 * 50000 - (50000 - 6282)
```

Warnings for
- `GapDetaViaDzerostarMixed/evtgen_exp_31_GapDetaViaDzerostarMixed-0.log` 50000
- `GapDetaViaDzerostarMixed/evtgen_exp_31_GapDetaViaDzerostarMixed-12.log` 50000
- `GapDetaViaDzerostarMixed/evtgen_exp_35_GapDetaViaDzerostarMixed-0.log` 50000
- `GapDetaViaDzerostarMixed/evtgen_exp_41b_GapDetaViaDzerostarMixed-12.log` 50000
- `GapDetaViaDzerostarMixed/evtgen_exp_47_GapDetaViaDzerostarMixed-32.log` 50000
- `GapDetaViaDzerostarMixed/evtgen_exp_51_GapDetaViaDzerostarMixed-22.log` 50000
- `GapDetaViaDzerostarMixed/evtgen_exp_55_GapDetaViaDzerostarMixed-64.log` 50000
- `GapDetaViaDzerostarMixed/evtgen_exp_63_GapDetaViaDzerostarMixed-31.log` 50000
- `GapDetaViaDzerostarMixed/evtgen_exp_65_GapDetaViaDzerostarMixed-40.log` 50000
- `GapDstetaViaDprimeoneMixed/evtgen_exp_19b_GapDstetaViaDprimeoneMixed-3.log` 50000
- `GapDstetaViaDprimeoneMixed/evtgen_exp_25_GapDstetaViaDprimeoneMixed-9.log` 50000
- `GapDstetaViaDprimeoneMixed/evtgen_exp_37b_GapDstetaViaDprimeoneMixed-21.log` 50000
- `GapDstetaViaDprimeoneMixed/evtgen_exp_41a_GapDstetaViaDprimeoneMixed-19.log` 50000
- `GapDstetaViaDprimeoneMixed/evtgen_exp_43_GapDstetaViaDprimeoneMixed-16.log` 50000
- `GapDstetaViaDprimeoneMixed/evtgen_exp_45a_GapDstetaViaDprimeoneMixed-5.log` 50000
- `GapDstetaViaDprimeoneMixed/evtgen_exp_51_GapDstetaViaDprimeoneMixed-16.log` 50000
- `GapDstetaViaDprimeoneMixed/evtgen_exp_55_GapDstetaViaDprimeoneMixed-27.log` 50000
- `GapDstetaViaDprimeoneMixed/evtgen_exp_65_GapDstetaViaDprimeoneMixed-13.log` 50000

**The Problematic Ones:**
```bash
gen: GapDEtaViaDstst/log/GapDstetaViaDprimeoneMixed/evtgen_exp_61_GapDstetaViaDprimeoneMixed-30.log  | 50000
sim: GapDEtaViaDstst/log/GapDstetaViaDprimeoneMixed/evtgen_exp_61_GapDstetaViaDprimeoneMixed-30.log  | 6282
```

### `GapDstPiPiViaDstst` (T) **RESUBMITTED**
Comparing
`/group/belle2/TMP/metzner/mc_production_august_2020/problem_check_mc_new_mixed_gen_summary_GapDstPiPiViaDstst_15thSep2020.log`
and
`/group/belle2/TMP/metzner/mc_production_august_2020/problem_check_mc_new_mixed_sim_summary_GapDstPiPiViaDstst_15thSep2020.log`

```python
(100000008 - 99891586) -2 * 50000 - (50000 - 41578)
```

Warnings for
- `GapDstpipiViaDoneprimeMixed/evtgen_exp_51_GapDstpipiViaDoneprimeMixed/evtgen_exp_51_GapDstpipiViaDoneprimeMixed-25.log` 50000
- `GapDstpipiViaDzerostarMixed/evtgen_exp_13_GapDstpipiViaDzerostarMixed/evtgen_exp_13_GapDstpipiViaDzerostarMixed-0.log` 50000

**The Problematic Ones:**
```bash
gen: GapDstpipiViaDzerostarMixed/evtgen_exp_61_GapDstpipiViaDzerostarMixed-18.log
sim: GapDstpipiViaDzerostarMixed/evtgen_exp_61_GapDstpipiViaDzerostarMixed-18.log
```


### Total Size on Discs

```python
(0.33 * 4 + 0.92 * 4 + 2.3 * 4 + 3.8 * 2 + 1.0 + 16 * 1.6) TB = 48.4 TB
```
Where `16 * 1.6 TB = 25.6 TB` are due to the `Dststlnu` sample alone and `22.8 TB` are due to the other samples.

**Moving some of the samples is necessary
due to the `35 TB` limit on the dedicated directory `/group/belle2/TMP/metzner`!**

**Hara-san and the KEKcc team extended the quota of `/group/belle2/TMP/metzner` to 60 TB!**

### Alternative Storage Location

**This section is no longer relevant after migration to new KEKcc!**

```bash
/gpfs/fs02/belle2/users/metzner/mc_production_extra_storage/
```
or
```bash
/group/belle2/users/metzner/mc_production_extra_storage/
```
which is the same location...

This is the location of the following samples:

| Data Type              | Original Size | Copied | GSim Check | Size Check | Files Check | `rsync` Check | All Checks | Deleted | Linked |
| :--------------------- | :------------ | :----- | ---------: | :--------- | :---------- | :------------ | :--------- | :------ | :----- |
| `Dlnu`                 | 314 GB        | Done   | True       | True       | True        | True          | True       | Yes     | Yes    |
| `Dstlnu`               | 330 GB        | Done   | True       | True       | True        | True          | True       | Yes     | Yes    |
| `DlnuMVA`              | 314 GB        | Done   | True       | True       | True        | True          | True       | Yes     | Yes    |
| `DstlnuMVA`            | 330 GB        | Done   | ERR but OK | True       | True        | True          | True       | Yes     | Yes    |
| `SignalDststlnupiz`    | 884 GB        | Done   | True       | True       | True        | True          | True       | Yes     | Yes    |
| `SignalDststlnupim`    | 910 GB        | Done   | True       | True       | True        | True          | True       | Yes     | Yes    |
| `SignalDststlnupizMVA` | 885 GB        | Done   | True       | True       | True        | True          | True       | Yes     | Yes    |
| `SignalDststlnupimMVA` | 910 GB        | Done   | True       | True       | True        | True          | True       | Yes     | Yes    |
| `GapDEta`              | 3.7 TB        | Done   | True       | True       | True        | True          | True       | Yes     | Yes    |
| `GapDPiPi`             | 440 GB        | Done   | True       | True       | True        | True          | True       | Yes     | Yes    |
| `GapDPiPiViaDstst`     | 559 GB        | Done   | True       | True       | True        | True          | True       | Yes     | Yes    |

`ERR but OK`: Here some `gsim` log files are already missing in the original directory.
Maybe the whole sample can be reproduced to be sure, but as the `.mdst` files are complete, we should be fine.

The samples are copied initially with the command

```bash
cp -a /group/belle2/TMP/metzner/mc_production_august_2020/<DataType> /gpfs/fs02/belle2/users/metzner/mc_production_extra_storage/<DataType>
```

and can be updated with

```bash
rsync -aunv /group/belle2/TMP/metzner/mc_production_august_2020/<DataType>/* /gpfs/fs02/belle2/users/metzner/mc_production_extra_storage/<DataType>/
```

in case the copy process fails due to the tight quota limit.

Finally, the following tests are performed to ensure completeness of the copied files:

- **(Size Check)** the directories copied are checked for matching size with `du -h`,
- **(Files Check)** the number of files and log files is checked with
    ```bash
    find /gpfs/fs02/belle2/users/metzner/mc_production_extra_storage/<DECAY_TYPE>//. ! -name . -print | grep -c //; find /group/belle2/TMP/metzner/mc_production_august_2020/<DECAY_TYPE>//. ! -name . -print | grep -c //;
    ```
- **(`rsync` Check)** a comparison with the `rsync`-based script `diff-dirs.sh` in the `mc_production` directory
    ```bash
    ./diff-dirs.sh <PATH_TO_OUTPUT_DIRECTORY>/<DECAY_TYPE> <PATH_TO_NEW_DIRECTORY>/<DECAY_TYPE>
    ```
    is performed, and
- **(GSim Check)** the `makeSummaryGsim` script
    ```bash
    ./gsim/makeSummaryGsim -MC <PATH_TO_NEW_DIRECTORY>/<DECAY_TYPE>/gsim/log/*/evtgen_exp_*-*[!gr].log  &> <PATH_TO_NEW_DIRECTORY>/mc_sim_summary_<DECAY_TYPE>_DDthMONTHYYYY.log
    ```
    is run again on the directories

**These files have been copied back to the original directory `/group/belle2/TMP/metzner/mc_production_august_2020/`
after the migration to the new KEKcc!**

# Migration Check off MC Production after KEKCC Upgrade on 1st of September 2020

The consistency of the MC files of the new production of August 2020 must be checked after it's migration to the new KEKCC system, 
as the KEKCC staff does not ensure the integrity of files produced or modified towards the end of August 2020.
Thus, checks of the data samples are performed using the following commands:

 ```bash
./diff-dirs.sh  ccold.cc.kek.jp:/group/belle2/TMP/metzner/ /group/belle2/TMP/metzner/
```
and
```bash
./diff-dirs.sh  ccold.cc.kek.jp:/group/belle2/users/metzner/mc_production_extra_storage/ /group/belle2/users/metzner/mc_production_extra_storage/
```

run from a server of the new system, and
where a `ssh` connection via `ccold.cc.kek.jp` allows temporary access to the old file system until the 11th September.

This check shows that there are files missing... of course...

The missing files are migrated afterwards using

```bash
rsync -auv ccold.cc.kek.jp:/group/belle2/TMP/metzner/<missing> /group/belle2/TMP/metzner/<missing>
```

from a server of the new system.

For a more detailed check, the following command can be run for specific directories:
```bash
rsync -acnuv ccold.cc.kek.jp:/group/belle2/TMP/metzner/mc_production_august_2020/Dststlnu/gsim/mdst/DprimeonezeroMu/ /group/belle2/TMP/metzner/mc_production_august_2020/Dststlnu/gsim/mdst/DprimeonezeroMu/
```
In this case the mdst files of the `Dststlnu` subsample `DprimeonezeroMu` are checked
also using a checksum (`-c`) for the consistency of their content.
The checksum option significantly increases the time required for the check.

**This final check was run after the missing files had been copied to the new KEKCC, confirming the consistency of the new samples.**


# Some Useful Commands

Look for `WARNING` in `gsim` summary log file:
```bash
grep -e "WAR" /group/belle2/TMP/metzner/mc_production_august_2020/mc_sim_summary_Dststlnu_DzerostarminusMu_10thAug2020.log
```

List Currently Running Jobs:
```bash
echo "GapDEtaViaDstst:"; bjobs | grep -e "Aug 20 00:5" | wc -l; echo "";echo "DzerostarzeroMu:"; bjobs | grep -e "Aug 20 03:" | wc -l;
```
