#!/usr/bin/python
from __future__ import absolute_import

import os
import sys
import shutil
import socket
import argparse
import subprocess


script_dir = sys.path[0]

dec_file_dirs = {
    "Dlnu": "{0}/dec_files/b_to_dlnu".format(script_dir),
    "Dstlnu": "{0}/dec_files/b_to_dstlnu".format(script_dir),
    "DlnuMVA": "{0}/dec_files/b_to_dlnu".format(script_dir),
    "DstlnuMVA": "{0}/dec_files/b_to_dstlnu".format(script_dir),
    "Dststlnu": "{0}/dec_files/b_to_dststlnu".format(script_dir),
    "SignalDststlnupiz": "{0}/dec_files/b_to_dststlnu_piz_signal".format(script_dir),
    "SignalDststlnupim": "{0}/dec_files/b_to_dststlnu_pim_signal".format(script_dir),
    "SignalDststlnupizMVA": "{0}/dec_files/b_to_dststlnu_piz_signal".format(script_dir),
    "SignalDststlnupimMVA": "{0}/dec_files/b_to_dststlnu_pim_signal".format(script_dir),
    "Dststtaunu": "{0}/dec_files/b_to_dststtaunu".format(script_dir),
    "SignalDststtaunupiz": "{0}/dec_files/b_to_dststtaunu_piz_signal".format(script_dir),
    "SignalDststtaunupim": "{0}/dec_files/b_to_dststtaunu_pim_signal".format(script_dir),
    "GapD1": "{0}/dec_files/b_to_gap_d1".format(script_dir),
    "GapDEta": "{0}/dec_files/b_to_gap_d_eta".format(script_dir),
    "GapDPiPi": "{0}/dec_files/b_to_gap_d_pipi".format(script_dir),
    "GapDstPiPi": "{0}/dec_files/b_to_gap_dst_pipi".format(script_dir),
    "GapDEtaViaDstst": "{0}/dec_files/b_to_gap_d_eta_via_dstst".format(script_dir),
    "GapDPiPiViaDstst": "{0}/dec_files/b_to_gap_d_pipi_via_dstst".format(script_dir),
    "GapDstPiPiViaDstst": "{0}/dec_files/b_to_gap_dst_pipi_via_dstst".format(script_dir),
}

mixed_dec_file_path_list = [
    "rdstar/mc_production/dec_files/b_to_dlnu/DminusE.dec",
    "rdstar/mc_production/dec_files/b_to_dlnu/DminusMu.dec",
    "rdstar/mc_production/dec_files/b_to_dlnu/DminusTau.dec",
    "rdstar/mc_production/dec_files/b_to_dstlnu/DstarminusE.dec",
    "rdstar/mc_production/dec_files/b_to_dstlnu/DstarminusMu.dec",
    "rdstar/mc_production/dec_files/b_to_dstlnu/DstarminusTau.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu/DoneminusE.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu/DoneminusMu.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu/DprimeoneminusE.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu/DprimeoneminusMu.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu/DtwostarminusE.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu/DtwostarminusMu.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu/DzerostarminusE.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu/DzerostarminusMu.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu_pim_signal/DoneminusE.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu_pim_signal/DoneminusMu.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu_pim_signal/DprimeoneminusE.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu_pim_signal/DprimeoneminusMu.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu_pim_signal/DtwostarminusE.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu_pim_signal/DtwostarminusMu.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu_pim_signal/DzerostarminusE.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu_pim_signal/DzerostarminusMu.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu_piz_signal/DoneminusE.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu_piz_signal/DoneminusMu.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu_piz_signal/DprimeoneminusE.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu_piz_signal/DprimeoneminusMu.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu_piz_signal/DtwostarminusE.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu_piz_signal/DtwostarminusMu.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu_piz_signal/DzerostarminusE.dec",
    "rdstar/mc_production/dec_files/b_to_dststlnu_piz_signal/DzerostarminusMu.dec",
    "rdstar/mc_production/dec_files/b_to_dststtaunu/DoneminusTau.dec",
    "rdstar/mc_production/dec_files/b_to_dststtaunu/DprimeoneminusTau.dec",
    "rdstar/mc_production/dec_files/b_to_dststtaunu/DtwostarminusTau.dec",
    "rdstar/mc_production/dec_files/b_to_dststtaunu/DzerostarminusTau.dec",
    "rdstar/mc_production/dec_files/b_to_dststtaunu_pim_signal/DoneminusTau.dec",
    "rdstar/mc_production/dec_files/b_to_dststtaunu_pim_signal/DprimeoneminusTau.dec",
    "rdstar/mc_production/dec_files/b_to_dststtaunu_pim_signal/DtwostarminusTau.dec",
    "rdstar/mc_production/dec_files/b_to_dststtaunu_pim_signal/DzerostarminusTau.dec",
    "rdstar/mc_production/dec_files/b_to_dststtaunu_piz_signal/DoneminusTau.dec",
    "rdstar/mc_production/dec_files/b_to_dststtaunu_piz_signal/DprimeoneminusTau.dec",
    "rdstar/mc_production/dec_files/b_to_dststtaunu_piz_signal/DtwostarminusTau.dec",
    "rdstar/mc_production/dec_files/b_to_dststtaunu_piz_signal/DzerostarminusTau.dec",
    "rdstar/mc_production/dec_files/b_to_gap_d1/GapD1Mixed.dec",
    "rdstar/mc_production/dec_files/b_to_gap_d_eta/GapDetaMixed.dec",
    "rdstar/mc_production/dec_files/b_to_gap_d_eta/GapDstetaMixed.dec",
    "rdstar/mc_production/dec_files/b_to_gap_d_eta_via_dstst/GapDetaViaDzerostarMixed.dec",
    "rdstar/mc_production/dec_files/b_to_gap_d_eta_via_dstst/GapDstetaViaDprimeoneMixed.dec",
    "rdstar/mc_production/dec_files/b_to_gap_d_pipi/GapDpipiMixed.dec",
    "rdstar/mc_production/dec_files/b_to_gap_d_pipi_via_dstst/GapDpipiViaDoneprimeMixed.dec",
    "rdstar/mc_production/dec_files/b_to_gap_d_pipi_via_dstst/GapDpipiViaDzerostarMixed.dec",
    "rdstar/mc_production/dec_files/b_to_gap_dst_pipi/GapDstpipiMixed.dec",
    "rdstar/mc_production/dec_files/b_to_gap_dst_pipi_via_dstst/GapDstpipiViaDoneprimeMixed.dec",
    "rdstar/mc_production/dec_files/b_to_gap_dst_pipi_via_dstst/GapDstpipiViaDzerostarMixed.dec",
    "rdstar/mc_production/dec_files/lus_new_b_to_gap_all/GapD1Mixed.dec",
    "rdstar/mc_production/dec_files/lus_new_b_to_gap_all/GapDprimeoneDetaMixed.dec",
    "rdstar/mc_production/dec_files/lus_new_b_to_gap_all/GapDprimeoneDpipiMixed.dec",
    "rdstar/mc_production/dec_files/lus_new_b_to_gap_all/GapDprimeoneDstpipiMixed.dec",
    "rdstar/mc_production/dec_files/lus_new_b_to_gap_all/GapDzerostarDetaMixed.dec",
    "rdstar/mc_production/dec_files/lus_new_b_to_gap_all/GapDzerostarDpipiMixed.dec",
    "rdstar/mc_production/dec_files/lus_new_b_to_gap_all/GapDzerostarDstpipiMixed.dec",
]

n_events_dict = {
    "Dlnu": 2500000,
    "Dstlnu": 2500000,
    "DlnuMVA": 2500000,
    "DstlnuMVA": 2500000,
    "Dststlnu": 73000000,
    "SignalDststlnupiz": 2500000,
    "SignalDststlnupim": 2500000,
    "SignalDststlnupizMVA": 2500000,
    "SignalDststlnupimMVA": 2500000,
    "Dststtaunu": 18000000,
    "SignalDststtaunupiz": 500000,
    "SignalDststtaunupim": 500000,
    "GapD1": 50000000,
    "GapDEta": 40000000,
    "GapDPiPi": 9500000,
    "GapDstPiPi": 46000000,
    "GapDEtaViaDstst": 40000000,
    "GapDPiPiViaDstst": 6000000,  # Twice as many dec files (D'_1, D_0*) as in GapDPiPi -> about half as many events!
    "GapDstPiPiViaDstst": 25000000,  # Twice as many dec files (D'_1, D_0*) as in GapDPiPi -> about half as many events!
}

if not set(dec_file_dirs.keys()) == set(n_events_dict.keys()):
    raise AssertionError("dec_file_dirs and n_events_dict have different keys!")


def check_host():
    """
    Checks if scripts are executed on the KEKCC. If not, the execution of the script is terminated.
    """
    if "cc.kek.jp" not in socket.gethostname():
        sys.exit("This script can only be executed on kekcc. Terminating!")
    if "BELLE_TOP_DIR" not in os.environ:
        sys.exit("You must source basf to run this script! Check the README if you don't know what to do...")
    if "BELLE2_LOCAL_DIR" in os.environ:
        sys.exit("If you want to run this script, basf2 must NOT be set up!")

class EvtGenerationManager:
    """
    Manages event generation with EvtGen:
      * copies and removes decay file to the runEvtgen directory
      * executes the runEvtgen script
    """
    def __init__(self, decay_key):
        """
        Constructor
        :param decay_key: Key of the decay process as given in the dec_file_dirs dictionary.
        :type decay_key: str
        """
        if not decay_key in dec_file_dirs.keys():
            raise AssertionError("decay_key {0} is not valid!".format(decay_key))

        self._decay_key = decay_key
        self._dec_file_dir = dec_file_dirs[self._decay_key]
        self._num_events_total = n_events_dict[self._decay_key]
        self._evtgen_dir = "{0}/evtgen".format(sys.path[0])
        self._dec_files = [
            d for d in os.listdir(self._dec_file_dir)
            if any(str(os.path.join(self._dec_file_dir, d)).endswith(f) for f in mixed_dec_file_path_list)
        ]

    def copy_dec_files_to_evtgen_dir(self):
        """
        Copies all decay file in the give directory to the evtgen directory.
        """
        for file in self._dec_files:
            shutil.copy("{0}/{1}".format(self._dec_file_dir, file),
                        "{0}/{1}".format(self._evtgen_dir, file))

    def tidy_up_evtgen_dir(self):
        """
        Deletes all decay files and job scripts in the given directory
        from the evtgen directory.
        """
        for file in self._dec_files:
            try:
                os.remove("{0}/{1}".format(self._evtgen_dir, file))
            except OSError:
                print "File {0}/{1} does not exist!".format(self._evtgen_dir, file)

        subprocess.call(["rm -f {0}/evtgen/*evtgen_exp*".format(sys.path[0])], shell=True)
        subprocess.call(["rm -f {0}/evtgen/*fpda_pid*".format(sys.path[0])], shell=True)

    def generate_events(self, job_num, test, output_path, clear_bad_mixed, clear_bad_mixed_dry_run):
        """
        Calls the ./evtgen/runEvtgen script with the given
        parameters. This creates basf scripts which are submitted
        to the LSF batch system. The generated events and log
        files are stored in the given output directory

        :param job_num: Maximum number of events submitted per LSF job.
        :type job_num: int
        :param test: Flag to run test run, which will ignore the job_num and total events
                     and produce a small sample of 1000 events with 10 jobs.
        :type test: bool
        :param output_path: Path to the output directory. Has to
        be of the for /abs/path/to/dir/. The last forward slash
        cannot be omitted.
        :type output_path: str
        :return:
        """
        safety_off = True

        target_path = str(os.path.join(output_path, self._decay_key)) + "/"

        evt_target_path = str(os.path.join(target_path, "evtgen")) + "/"

        os.chdir(os.path.join(sys.path[0], "evtgen"))

        n_events_total = str(self._num_events_total) if not test else "1000"
        n_events_per_job = str(job_num) if not test else "100"

        if not os.path.exists(evt_target_path):
            os.makedirs(evt_target_path)

        for dec_file in self._dec_files:
            if clear_bad_mixed or clear_bad_mixed_dry_run:
                d_prefix = "false_mixing_"
                sub_dir_strings = [
                    "gsim/log",
                    "gsim/hbook",
                    "gsim/mdst",
                    "evtgen/log",
                    "evtgen/gen"
                ]
                for sub_dir_str in sub_dir_strings:
                    old_result_path = os.path.join(target_path, sub_dir_str, os.path.splitext(dec_file)[0])
                    new_result_path = os.path.join(target_path, sub_dir_str, d_prefix + os.path.splitext(dec_file)[0])
                    if os.path.exists(old_result_path):
                        if clear_bad_mixed_dry_run:
                            print "Would remove\n\t{}".format(old_result_path)
                            # print "Would rename\n\t{}\nto\n\t{}".format(old_result_path, new_result_path)
                        elif clear_bad_mixed and safety_off and (not clear_bad_mixed_dry_run):
                            shutil.rmtree(old_result_path)
                            print "Removed  {}".format(old_result_path)
                            # os.rename(old_result_path, new_result_path)
                            # print "Renamed {}  to  {}".format(old_result_path, new_result_path)
                            if os.path.exists(old_result_path):
                                raise RuntimeError("Directory {} could not be removed!".format(old_result_path))
                        else:
                            raise RuntimeError(
                                "clear_bad_mixed {}, safety_off {}, clear_bad_mixed_dry_run {}".format(
                                    clear_bad_mixed, safety_off, clear_bad_mixed_dry_run
                                )
                            )
            else:
                print "\n\n"
                print "\tGenerating {0} events ({1} per job) for decay file {2}/{3}".format(
                    n_events_total, n_events_per_job, self._dec_file_dir, dec_file)
                print "\tOutput will be stored in {0}".format(evt_target_path)
                print "\n\n"
                subprocess.check_call([
                    "./runEvtgen",
                    "nBB-Y4S.txt",
                    dec_file,
                    "Y4S.conf",
                    n_events_total,
                    n_events_per_job,
                    evt_target_path
                ])


def parse_command_line_args():
    """
    Parses command line arguments.

    :return: Command line arguments.
    """

    parser = argparse.ArgumentParser(description="This script starts the event generation with EvtGen for the given"
                                                 " Decay type.")

    parser.add_argument("-t", action="store", dest="dec_type", type=str, required=True,
                        help="Decay type: Currently available is " + ", ".join(dec_file_dirs.keys()))
    parser.add_argument("-o", action="store", dest="out_path", type=str, required=True,
                        help="Absolute output path for the generated events. "
                             "Has to end with a /. "
                             "Example: '/group/belle2/users/welsch/test_mc/evtgen'")
    parser.add_argument("-j", action="store", dest="job_num", type=int, required=True,
                        help="Maximum number of events per job.")

    parser.add_argument("--test", action="store_true", required=False,
                        help="Test production of 1000 events with 100 events per job.")
    parser.add_argument("--clean", action="store_true", required=False,
                        help="Tidy up the evtgen directory after all jobs ran successfully.")
    parser.add_argument("--clear_bad_mixed_dry_run", action="store_true", required=False,
                        help="Dry run for removal of every product of mixed decay files.")
    parser.add_argument("--clear_bad_mixed", action="store_true", required=False,
                        help="Remove every product of mixed decay files.")

    parsed_args = parser.parse_args()

    return parsed_args

if __name__ == "__main__":
    check_host()

    args = parse_command_line_args()

    if args.dec_type not in dec_file_dirs.keys():
        raise ValueError("Specified decay type not known!")

    evtgen_manager = EvtGenerationManager(decay_key=args.dec_type)

    if not args.clean:
        evtgen_manager.copy_dec_files_to_evtgen_dir()

        evtgen_manager.generate_events(
            job_num=args.job_num,
            test=args.test,
            output_path=args.out_path,
            clear_bad_mixed=args.clear_bad_mixed,
            clear_bad_mixed_dry_run=args.clear_bad_mixed_dry_run
        )
    else:
        evtgen_manager.tidy_up_evtgen_dir()



