"""
Definition of MVA Method for Event Selection Optimization

Patrick Ecker 2020
"""

import os
from typing import Union, Optional, Tuple, List

from rdstar.mva_methods.mva_methods import MVABaseClass
from rdstar.mva_methods.mva_validation import MVAValidation
from rdstar.mva_selection.mva_selector_setups import MVASelectorSetups

__all__ = [
    "MVASelector",
    "MVASelectorValidation"
]


class MVASelector(MVABaseClass):
    def __init__(
            self,
            selection_mva_variables: List[str],
            target_variable: str,
            weight: Optional[str] = None
    ) -> None:
        super().__init__(
            variables=selection_mva_variables,
            target_variable=target_variable,
            weight=weight
        )


class MVASelectorValidation(MVAValidation):
    def __init__(
            self,
            path_train_df: Union[str, os.fspath],
            path_test_df: Union[str, os.fspath],
            path_mva_var_list: Union[str, os.fspath],
            path_mva_bdt_weights: Union[str, os.fspath],
            mva_selector_setup_name: str,
            target_dir_path: Union[str, os.fspath]
    ) -> None:
        self._mva_setup = MVASelectorSetups.get_selection_mva_setup(setup_name=mva_selector_setup_name)

        super().__init__(
            mva_class=MVASelector,
            path_train_df=path_train_df,
            path_test_df=path_test_df,
            path_var_list=path_mva_var_list,
            path_bdt_weights=path_mva_bdt_weights,
            target_var=self._mva_setup.get_target_column_name(),
            training_histogram_variables_dict=self._mva_setup.get_selection_variable_dict(),
            mva_output_name=self._mva_setup.get_selection_mva_output_column_name(),
            output_dir_path=target_dir_path
        )

    def run_validation(
            self,
            base_name: str,
            working_point: float,
            dry_run: bool = False,
            mva_output_latex_name: Optional[str] = None
    ) -> Tuple[List[Tuple[str, Union[str, os.fspath]]], float, float]:
        return super().run_validation(
            mva_output_latex_name="Event Selection MVA" if mva_output_latex_name is None else mva_output_latex_name,
            base_name=base_name,
            working_point=working_point,
            dry_run=dry_run
        )
