# Offline Tasks

## How to Run the Offline Tasks
Just execute the command

```shell
python3 rdstar/offline_analysis/offline_tasks.py &> </path/to/some/dir/>offline_tasks_DDthMONTHYYYY.log
```

preferably in a shell session with your environment set up. Of course the log file is optional.

## Task Tree

 1.0 DataCopyTask

        New Parameters:
            - `offline_config_path`
 
 2.0 DataInitialConversionTask @requires DataCopyTask

        New Parameters:
            - `data_type`
            - `stream`
            - `tuple_type`

 3.0 DataPreparationSplitterTask @requires DataInitialConversionTask

        New Parameters:
            - `n_subsets`

 4.0 DataPreparationTask @inherits DataPreparationSplitterTask

        -> requires DataPreparationSplitterTask

        New Parameters:
            - `subset_index` in `[0, n_subsets]`

 5.0 DataPreparationMergerTask @inherits and requires DataPreparationTask for each subset_index

        No new Parameters

 6.0 CSDataPreparationTask @inherits DataCopyTask

        -> requires DataPreparationMergerTask

        New Parameters:
            - `cs_data_tag`
            - `cs_use_signal_side_vars`

 7.0 CSTrainTask @ requires CSDataPreparationTask

        New Parameters:
            - `cs_tag`
            - `cs_fastbdt_settings`

 8.1 CSTestTask @inherits DataCopyTask

        -> requires CSTrainTask, CSDataPreparationTask

        New Parameters:
            - `cs_tag`
            - `cs_fastbdt_settings`

 8.2 DataReductionTask @inherits DataPreparationSplitterTask, CSTestTask

        -> requires CSTestTask, DataPreparationMergerTask

        No new Parameters

 9.0 DataSelectionTask @requires DataReductionTask

        New Parameters:
            - `selection_name`

 10.1 SampleCombinerTask

    -> requires DataSelectionTask, DataPreparationMergerTask

    New Parameter:
        - `combo_name`
        - `working_dir`
        - `selection_name`
        - `config_params`

 10.2 SelectionInfoCombinerTask

    -> requires DataSelectionTask for each `config_combination`

    New Parameter:
        - `working_dir`
        - `selection_name`
        - `config_combinations`


 10.3. EfficiencyEvaluationTask @inherits WeightAnalysisTask

    -> requires DataSelectionTask

    No new Parameters

 11.1 AnalysisEvaluationTask @requires SelectionInfoCombinerTask

    New Parameters:
        - `cs_tag`

 11.2 WeightAnalysisTask

    -> requires SampleCombinerTask

    New Parameters:
        - `working_dir`
        - `selection_name`
        - `cs_tag`
        - `config_combinations`

 12.0 WeightFinalizationTask @inherits SampleCombinerTask, WeightAnalysisTask

    -> requires SampleCombinerTask, WeightAnalysisTask

    No new Parameters
    
 13.0 ResolutionEvaluationTask @inherits WeightAnalysisTask
 
    -> requires WeightFinalizationTask
 
    No new Parameters
 
 14.0 ResolutionCorrectionTask @inherits WeightFinalizationTask
 
    -> requires WeightFinalizationTask, ResolutionEvaluationTask
 
    No new Parameters

 15.1 SidebandEvaluationTask @inherits WeightAnalysisTask

    -> requires ResolutionCorrectionTask

    No new Parameters

 15.2 ResolutionCorrectionValidationTask @inherits ResolutionEvaluationTask

    -> requires ResolutionCorrectionTask

    No new Parameters

 16.0 FitTask @inherits ResolutionCorrectionTask

    -> requires ResolutionCorrectionTask

    New Parameters:
        - `fit_case_key`

 17.0 FittingTasksWrapperTask @inherits WeightAnalysisTask

    -> requires FitTask for fit_case_key in rdstar_fit_models.rdstar_fit_cases.keys()

    No new Parameters

 18.0 ValidationWrapperTask @requires AnalysisEvaluationTask, SidebandEvaluationTask, ResolutionCorrectionValidationTask

    No new Parameters

 19.0 OfflineAnalysisMaster

    -> requires ResolutionCorrectionTask, ValidationWrapperTask, EfficiencyEvaluationTask
    TODO: Maybe replace WeightFinalizationTask with ResolutionCorrectionTask here!

    New Parameters:
        - `offline_config_paths`
