"""
Convenience functions to add K short correction weights.

Felix Metzner 2019
"""

import pandas as pd

from analysistools.k_short_correction.k_short_correction import KShortCorrection

from rdstar.offline_analysis.corrections.particle_id_corrections import ParticleIDCorrection


class KShortEffCorrector:
    k_short_corr_col_name = "__weight_k_short_ratio__"
    k_short_corr_var_prefix = "__weight_k_short_var_"

    def __init__(self):
        self.exp_column_name = "expNum"

        self.k_short_corrector = KShortCorrection(exp_column_name=self.exp_column_name)

        for name, track_info in self.get_track_infos().items():
            self.k_short_corrector.add_track(name=name, **track_info)

    def apply_k_short_correction(self, sample: pd.DataFrame):
        for col in [c for infos in self.get_track_infos().values() for c in infos.values()]:
            assert col in sample.columns, f"{col}"

        k_short_corr = self.k_short_corrector.get_corrections(df=sample)

        for col in k_short_corr.columns:
            assert col not in sample.columns, f"{col}"
        assert len(k_short_corr) == len(sample.index), f"{len(k_short_corr)} and {len(sample.index)}"

        for col in k_short_corr.columns:
            sample.loc[:, col] = k_short_corr[col].values

    @staticmethod
    def get_track_infos():
        # We only reconstruct K shorts as first daughter (-> index 0) of D or D* mesons.
        track_infos = {
            "D0": {
                "pdg_column": ParticleIDCorrection.get_d_daughter_var(daughter=0, variable="PDG"),
                "momentum_column": ParticleIDCorrection.get_d_daughter_var(daughter=0, variable="p"),
            },
            "Dst0": {
                "pdg_column": ParticleIDCorrection.get_dd_daughter_var(daughter=0, variable="PDG"),
                "momentum_column": ParticleIDCorrection.get_dd_daughter_var(daughter=0, variable="p"),
            }}

        return track_infos
