"""
Continuum Suppression MVA Validation

Felix Metzner 2020
"""

import os

from typing import Union, Optional, Tuple, List

from rdstar.mva_methods.mva_validation import MVAValidation
from rdstar.continuum_suppression import ContinuumSuppression, get_cs_variable_dict

__all__ = [
    "CSValidation"
]


class CSValidation(MVAValidation):
    def __init__(
            self,
            path_train_df: Union[str, os.fspath],
            path_test_df: Union[str, os.fspath],
            path_cs_var_list: Union[str, os.fspath],
            path_cs_bdt_weights: Union[str, os.fspath],
            cs_output_col_name: str,
            cs_target_var: str,
            target_dir_path: Union[str, os.fspath],
            using_sig_side_vars: bool = True,
    ) -> None:
        cs_hist_variable_dict = get_cs_variable_dict(signal_side=using_sig_side_vars)

        super().__init__(
            mva_class=ContinuumSuppression,
            path_train_df=path_train_df,
            path_test_df=path_test_df,
            path_var_list=path_cs_var_list,
            path_bdt_weights=path_cs_bdt_weights,
            target_var=cs_target_var,
            training_histogram_variables_dict=cs_hist_variable_dict,
            mva_output_name=cs_output_col_name,
            output_dir_path=target_dir_path
        )

    def run_validation(
            self,
            base_name: str,
            working_point: float,
            dry_run: bool = False,
            mva_output_latex_name: Optional[str] = None
    ) -> Tuple[List[Tuple[str, Union[str, os.fspath]]], float, float]:
        return super().run_validation(
            mva_output_latex_name="Continuum Suppression" if mva_output_latex_name is None else mva_output_latex_name,
            base_name=base_name,
            working_point=working_point,
            dry_run=dry_run
        )
