"""
This file contains all the best candidate selection settings relevant to the analysis.

The convention for the form of the best candidate selection settings are explained
in best_cand_selection_utils.py

It is important to note, that the final Y(4S) particle is always
reconstructed in such a way, that the first daughter (index 0) is
the tag B meson, and the second (index 1) is the signal B meson.

Felix Metzner 2018
"""

from analysistools.processing.best_cand_selection_utils import print_bcs_settings


upsilon4s_with_fei_probability_bcs = {
    "particle": "Upsilon(4S)",
    "rank_by": "highest",
    "variable": "daughter(0, extraInfo(SignalProbability))",
    "output_variable_name": "feiTagSideSignalProbabilityRank",
    "num_of_cands": 0,
    "multi_rank": True,
}

upsilon4s_with_vertex_fit_probability_bcs = {
    "particle": "Upsilon(4S)",
    "rank_by": "highest",
    "variable": "daughter(1, chiProb)",
    "output_variable_name": "vertexFitChiProbabilityRank",
    "num_of_cands": 0,
    "multi_rank": True,
}

upsilon4s_with_roe_eextra_bcs = {
    "particle": "Upsilon(4S)",
    "rank_by": "lowest",
    "variable": "ROE_eextra(CleanROE)",
    "output_variable_name": "roeEExtraRank",
    "num_of_cands": 0,
    "multi_rank": True,
}

upsilon4s_with_roe_neextra_bcs = {
    "particle": "Upsilon(4S)",
    "rank_by": "lowest",
    "variable": "ROE_neextra(CleanROE)",
    "output_variable_name": "roeNEExtraRank",
    "num_of_cands": 0,
    "multi_rank": True,
}

upsilon4s_with_deltaE_bcs = {
    "particle": "Upsilon(4S)",
    "rank_by": "lowest",
    "variable": "abs(daughter(0, deltaE))",
    "output_variable_name": "deltaERank",
    "num_of_cands": 0,
    "multi_rank": True,
}

upsilon4s_with_DMeson_deltaM_bcs = {
    "particle": "Upsilon(4S)",
    "rank_by": "lowest",
    "variable": "abs(daughter(0, daughter(0, dM)))",
    "output_variable_name": "U4SDMesonDeltaMRank",
    "num_of_cands": 0,
    "multi_rank": True,
}

D_meson_by_dM_bcs = {
    "particle": "D",
    "rank_by": "lowest",
    "variable": "abs(dM)",
    "output_variable_name": "DMesonDeltaMRank",
    "num_of_cands": 10,
    "multi_rank": False,
}

my_bcs_settings = {
    "upsilon4s_with_fei_probability_bcs": upsilon4s_with_fei_probability_bcs,
    "D_meson_by_dM_bcs": D_meson_by_dM_bcs
}

if __name__ == "__main__":
    print_bcs_settings(my_bcs_settings)
