"""
Steering file for testing the randomness of gsim.

Felix Metzner 2020
"""

import basf2
import b2biiConversion

from modularAnalysis import *

from analysistools.processing import path_utilities

from rdstar.online_analysis.decay_utilities import Reconstructor
from rdstar.online_analysis import observables, decay_utilities, decay_channels


config = {
    "use_mc_info": True,
    "roe_mask": "CleanROE",
    "own_local_mc_db_path": "/home/belle2/metzner/databases/RDStar_b2bii/mc_beamparams/localdb/",
    "custom_fei_db_path": None,
    "FEI_options": {
        "FEI_tag": "FEIv4_2017_MCConverted_Track14_2",
        "hadronic": True,
        "semileptonic": False,
        "specific": False
    },
    "use_b2bii": True
}

input_dict = {
    "old_exp55_out": "/group/belle2/TMP/metzner/mc_production_august_2020/GapDPiPiViaDstst/gsim/mdst/GapDpipiViaDoneprimeCharged/evtgen_exp_55_GapDpipiViaDoneprimeCharged-1.mdst",
    # "new1_exp55_out": "/home/belle2/rvantond/temp_resubmitting_one_mc_job/gsim/mdst/GapDpipiViaDoneprimeCharged/evtgen_exp_55_GapDpipiViaDoneprimeCharged-1.mdst",
    # "new2_exp55_out": "/home/belle2/rvantond/temp_resubmitting_one_mc_job_take2/gsim/mdst/GapDpipiViaDoneprimeCharged/evtgen_exp_55_GapDpipiViaDoneprimeCharged-1.mdst",
    # "new3_exp55_out": "/home/belle2/rvantond/temp_resubmitting_one_mc_job_take3/gsim/mdst/GapDpipiViaDoneprimeCharged/evtgen_exp_55_GapDpipiViaDoneprimeCharged-1.mdst",
    # "new1_exp19a_out": "/home/belle2/rvantond/temp_resubmitting_one_mc_job/gsim/mdst/GapDpipiViaDoneprimeCharged/evtgen_exp_19a_GapDpipiViaDoneprimeCharged-1.mdst",
    # "new2_exp19a_out": "/home/belle2/rvantond/temp_resubmitting_one_mc_job_take2/gsim/mdst/GapDpipiViaDoneprimeCharged/evtgen_exp_19a_GapDpipiViaDoneprimeCharged-1.mdst",
    # "new3_exp19a_out": "/home/belle2/rvantond/temp_resubmitting_one_mc_job_take3/gsim/mdst/GapDpipiViaDoneprimeCharged/evtgen_exp_19a_GapDpipiViaDoneprimeCharged-1.mdst",
}

assert len(input_dict) == 1, input_dict

input_files = [v for v in input_dict.values()]
input_key = str(list(input_dict.keys())[0])
assert len(input_files) == 1, (input_files, input_key, input_dict)

target_dir_path = "/home/belle2/metzner/temp_gsim_test/"

output_dict = {
    "piNeutral_reco_vertex_ntuple": os.path.join(target_dir_path, f"{input_key}_piNeutral_reco_vertex_ntuple.root"),
    "piNeutral_reco_ntuple": os.path.join(target_dir_path, f"{input_key}_piNeutral_reco_ntuple.root"),
    "piNeutral_from_list_ntuple": os.path.join(target_dir_path, f"{input_key}_piNeutral_from_list_ntuple.root"),
    "e_ntuple": os.path.join(target_dir_path, f"{input_key}_e_ntuple.root"),
    "mu_ntuple": os.path.join(target_dir_path, f"{input_key}_mu_ntuple.root"),
    "piCharged_ntuple": os.path.join(target_dir_path, f"{input_key}_piCharged_ntuple.root"),
    "B_zero_generator_direct_info_ntuple": os.path.join(target_dir_path, f"{input_key}_B_zero_generator_direct_info_ntuple.root"),
    "B_minus_generator_direct_info_ntuple": os.path.join(target_dir_path, f"{input_key}_B_minus_generator_direct_info_ntuple.root"),
    "gamma_ntuple": os.path.join(target_dir_path, f"{input_key}_gamma_ntuple.root"),
    "K_ntuple": os.path.join(target_dir_path, f"{input_key}_K_ntuple.root"),
    "K_S0_ntuple": os.path.join(target_dir_path, f"{input_key}_K_S0_ntuple.root"),
}

path_utilities.setup_environment_vars()
path_utilities.setup_databases(config)
path_utilities.fei_backward_compatibility()

main_path = basf2.create_path()

b2biiConversion.convertBelleMdstToBelleIIMdst(
    inputBelleMDSTFile=input_files,
    applyHadronBJSkim=True,
    useBelleDBServer=os.environ["BELLE_POSTGRES_SERVER"],
    path=main_path,
)

setAnalysisConfigParams({"mcMatchingVersion": "Belle"}, main_path)

main_path.add_module("CurlerCloneMarker")
decay_utilities.fsp.fsp_collection.get_final_state_particles(path=main_path, use_mc_info=config["use_mc_info"])

reconstructor = Reconstructor(path=main_path, config=config, decay_channels_module=decay_channels)

#######
# pi0 #
#######

piNeutralVars = [
    "PDG",
    "matchedMC(PDG)",
    "mcMother(PDG)",
    "mcMother(mcMother(PDG))",
    "mcMother(mcMother(mcMother(PDG)))",
    "mcMother(mcMother(mcMother(mcMother(PDG))))",
    "mcMother(mcDaughter(0,PDG))",
    "isSignal",
    "M", "dM", "InvM",
    "E",
    "Q", "dQ",
    "useCMSFrame(p)",
    "useCMSFrame(pt)",
    "daughterInvariantMass(0, 1)",
    "chiProb",
    "abs(SigMBF)",
    "daughterAngle(0,1)",
    "useRestFrame(daughterAngle(0,1))",
    "daughterAngleInBetween(0,1)",
    "useRestFrame(daughterAngleInBetween(0,1))",
    "p", "pt", "px", "py", "pz",
    "daughter(0, goodBelleGamma)",
    "daughter(0, clusterE9E21)",
    "daughter(0, clusterE1E9)",
    "daughter(0, clusterE9E25)",
    "daughter(0, clusterNHits)",
    "daughter(0, clusterHighestE)",
    "daughter(0, E)",
    "daughter(1, goodBelleGamma)",
    "daughter(1, clusterE9E25)",
    "daughter(1, clusterNHits)",
    "daughter(1, clusterHighestE)",
    "daughter(1, E)",
    "eIDBelle",
    "muIDBelle",
    "muIDBelleQuality",
    "atcPIDBelle(2,3)",
    "extraInfo(ksnbStandard)",
    "extraInfo(goodKs)",
    "dr", "dz",
    "daughterDiffOf(0, 1, E)",
    "daughterSumOf(E)",
    "formula(daughterDiffOf(0, 1, E) / daughterSumOf(E))"
]

main_path.add_module(
    "VariablesToNtuple",
    fileName=output_dict["piNeutral_from_list_ntuple"],
    particleList="pi0:fsp",
    variables=piNeutralVars
)

reconstructor.reconstruct_pi0()

main_path.add_module(
    "VariablesToNtuple",
    fileName=output_dict["piNeutral_reco_ntuple"],
    particleList="pi0:reco",
    variables=piNeutralVars
)

massVertexKFit(
    list_name="pi0:reco",
    conf_level=-2.,
    decay_string="",
    path=main_path,
)

main_path.add_module(
    "VariablesToNtuple",
    fileName=output_dict["piNeutral_reco_vertex_ntuple"],
    particleList="pi0:reco",
    variables=piNeutralVars
)

#######
# pi+ #
#######

piChargedVars = [
    "PDG",
    "matchedMC(PDG)",
    "mcMother(PDG)",
    "mcMother(mcMother(PDG))",
    "mcMother(mcMother(mcMother(PDG)))",
    "mcMother(mcMother(mcMother(mcMother(PDG))))",
    "mcMother(mcDaughter(0,PDG))",
    "isSignal",
    "M", "dM", "InvM",
    "E",
    "Q", "dQ",
    "useCMSFrame(p)",
    "useCMSFrame(pt)",
    "p", "pt", "px", "py", "pz",
    "eIDBelle",
    "muIDBelle",
    "muIDBelleQuality",
    "atcPIDBelle(2,3)",
    "extraInfo(ksnbStandard)",
    "extraInfo(goodKs)",
    "dr", "dz",
    "trackCurlerCloneIndicator"
]

main_path.add_module(
    "VariablesToNtuple",
    fileName=output_dict["piCharged_ntuple"],
    particleList="pi+:fsp",
    variables=piChargedVars
)

############
# e and mu #
############

leptonVars = [
    "PDG",
    "matchedMC(PDG)",
    "mcMother(PDG)",
    "mcMother(mcMother(PDG))",
    "mcMother(mcDaughter(0,PDG))",
    "mcDaughter(0,PDG)",
    "isSignal",
    "M", "dM", "InvM",
    "daughterInvariantMass(0, 1)",
    "E",
    "Q", "dQ",
    "useCMSFrame(p)",
    "useCMSFrame(pt)",
    "p", "pt", "px", "py", "pz",
    "eIDBelle",
    "muIDBelle",
    "muIDBelleQuality",
    "atcPIDBelle(2,3)",
    "extraInfo(ksnbStandard)",
    "extraInfo(goodKs)",
    "dr", "dz",
    "pValue",
    "nCDCHits",
    "nSVDHits",
    "daughter(0, pValue)",
    "daughter(0, nCDCHits)",
    "daughter(0, nSVDHits)",
    "daughter(0, dr)",
    "daughter(0, dz)",
    "matchedMC(E)",
    "matchedMC(p)",
    "matchedMC(px)",
    "matchedMC(py)",
    "matchedMC(pz)",
    "trackCurlerCloneIndicator",
    "extraInfo(fsrCorrectedE)",
    "extraInfo(fsrCorrectedPx)",
    "extraInfo(fsrCorrectedPy)",
    "extraInfo(fsrCorrectedPz)",
    "extraInfo(fsrCorrectedP)",
    "extraInfo(fsrGammaE)",
    "extraInfo(fsrGammaPx)",
    "extraInfo(fsrGammaPy)",
    "extraInfo(fsrGammaPz)",
    "extraInfo(fsrGammaP)",
    "extraInfo(fsrCorrected)",
    "Eher",
    "Eler",
    "Ecms",
    "XAngle",
    "beamE",
    "beamPx",
    "beamPy",
    "beamPz",
    "expNum",
    "evtNum",
    "runNum",
    "nTracks",
    "nECLClusters",
    "belleECLEnergy",
    "nKLMClusters",
    "KLMEnergy",
    "nMCParticles",
    "IPX",
]


main_path.add_module(
    "VariablesToNtuple",
    fileName=output_dict["e_ntuple"],
    particleList="e-:fsp",
    variables=leptonVars
)

main_path.add_module(
    "VariablesToNtuple",
    fileName=output_dict["mu_ntuple"],
    particleList="mu-:fsp",
    variables=leptonVars
)

########
# K_S0 #
########

KS0Vars = [
    "PDG",
    "matchedMC(PDG)",
    "mcMother(PDG)",
    "mcMother(mcMother(PDG))",
    "mcMother(mcMother(mcMother(PDG)))",
    "mcMother(mcMother(mcMother(mcMother(PDG))))",
    "mcMother(mcDaughter(0,PDG))",
    "isSignal",
    "M", "dM", "InvM",
    "abs(SigMBF)",
    "E",
    "Q", "dQ",
    "useCMSFrame(p)",
    "useCMSFrame(pt)",
    "p", "pt", "px", "py", "pz",
    "eIDBelle",
    "muIDBelle",
    "muIDBelleQuality",
    "atcPIDBelle(2,3)",
    "extraInfo(ksnbStandard)",
    "extraInfo(goodKs)",
    "dr", "dz"
]

main_path.add_module(
    "VariablesToNtuple",
    fileName=output_dict["K_S0_ntuple"],
    particleList="K_S0:fsp",
    variables=KS0Vars
)

######
# K+ #
######

KVars = [
    "PDG",
    "matchedMC(PDG)",
    "mcMother(PDG)",
    "mcMother(mcMother(PDG))",
    "mcMother(mcMother(mcMother(PDG)))",
    "mcMother(mcMother(mcMother(mcMother(PDG))))",
    "mcMother(mcDaughter(0,PDG))",
    "isSignal",
    "M", "dM", "InvM",
    "E",
    "Q", "dQ",
    "useCMSFrame(p)",
    "useCMSFrame(pt)",
    "p", "pt", "px", "py", "pz",
    "eIDBelle",
    "muIDBelle",
    "muIDBelleQuality",
    "atcPIDBelle(2,3)",
    "extraInfo(ksnbStandard)",
    "extraInfo(goodKs)",
    "dr", "dz",
    "trackCurlerCloneIndicator"
]

main_path.add_module(
    "VariablesToNtuple",
    fileName=output_dict["K_ntuple"],
    particleList="K+:fsp",
    variables=KVars
)

#########
# Gamma #
#########

gammaVars = [
    "PDG",
    "matchedMC(PDG)",
    "mcMother(PDG)",
    "mcMother(mcMother(PDG))",
    "mcMother(mcMother(mcMother(PDG)))",
    "mcMother(mcMother(mcMother(mcMother(PDG))))",
    "mcMother(mcDaughter(0,PDG))",
    "isSignal",
    "goodBelleGamma",
    "clusterE9E25",
    "clusterNHits",
    "clusterHighestE",
    "M", "dM", "InvM",
    "abs(SigMBF)",
    "E",
    "Q", "dQ",
    "useCMSFrame(p)",
    "useCMSFrame(pt)",
    "p", "pt", "px", "py", "pz",
    "eIDBelle",
    "muIDBelle",
    "muIDBelleQuality",
    "atcPIDBelle(2,3)",
    "extraInfo(ksnbStandard)",
    "extraInfo(goodKs)",
    "dr", "dz",
    "matchedMC(E)",
    "matchedMC(px)",
    "matchedMC(py)",
    "matchedMC(pz)",
    "matchedMC(p)",
    # "extraInfo(fsrCorrected)"
]

main_path.add_module(
    "VariablesToNtuple",
    fileName=output_dict["gamma_ntuple"],
    particleList="gamma:fsp",
    variables=gammaVars
)

if config["use_mc_info"] is True:
    b_direct_gen_coll = observables.get_direct_gen_level_b_collection(config)

    fillParticleListsFromMC([("B-:mc", "")], addDaughters=True, path=main_path)
    fillParticleListsFromMC([("B0:mc", "")], addDaughters=True, path=main_path)
    main_path.add_module(
        "VariablesToNtuple",
        fileName=output_dict["B_minus_generator_direct_info_ntuple"],
        particleList="B-:mc",
        variables=b_direct_gen_coll.as_list()
    )
    main_path.add_module(
        "VariablesToNtuple",
        fileName=output_dict["B_zero_generator_direct_info_ntuple"],
        particleList="B0:mc",
        variables=b_direct_gen_coll.as_list()
    )

basf2.print_path(main_path)
basf2.process(main_path)
print(statistics)
