"""
This file updates the decay channels cut infos for the low level cut study.

Patrick Ecker 2020
"""

import copy

from rdstar.online_analysis.decay_channels import *
from rdstar.online_analysis.decay_channels import \
    D_decay_label_to_cut_info_list as _original_D_decay_label_to_cut_info_list
from rdstar.online_analysis.decay_channels import \
    Dstar_decay_label_to_cut_info_list as _original_Dstar_decay_label_to_cut_info_list
from rdstar.online_analysis.decay_channels import decay_channels_utils, channels_with_neutrinos, \
    generator_level_only_channels

__all__ = [
    "decay_channel_dict",
    "D_decay_label_to_cut_info_list",
    "D_decay_label_to_cut_list",
    "Dstar_decay_label_to_cut_info_list",
    "Dstar_decay_label_to_cut_list"
]

# Update the D cuts
D_decay_label_to_cut_info_list = copy.copy(_original_D_decay_label_to_cut_info_list)
for channel, cut_infos in D_decay_label_to_cut_info_list.items():
    D_decay_label_to_cut_info_list[channel] = [cut_info.scale_cut(percentage=1.) for cut_info in cut_infos]
    # D_decay_label_to_cut_info_list[channel] = []

D_decay_label_to_cut_list = {}
for channel, cut_infos in D_decay_label_to_cut_info_list.items():
    D_decay_label_to_cut_list[channel] = [cut_info.cut_str for cut_info in cut_infos]

# Update the D* cuts
Dstar_decay_label_to_cut_info_list = copy.copy(_original_Dstar_decay_label_to_cut_info_list)
for channel, cut_infos in Dstar_decay_label_to_cut_info_list.items():
    Dstar_decay_label_to_cut_info_list[channel] = [cut_info.scale_cut(percentage=1.) for cut_info in cut_infos]
    # Dstar_decay_label_to_cut_info_list[channel] = []

Dstar_decay_label_to_cut_list = {}
for channel, cut_infos in Dstar_decay_label_to_cut_info_list.items():
    Dstar_decay_label_to_cut_list[channel] = [cut_info.cut_str for cut_info in cut_infos]

decay_channel_dict = decay_channels_utils.prepare_decay_channel_dict(
    local_vars=locals(),
    channels_with_neutrinos=channels_with_neutrinos,
    gen_level_only_channels=generator_level_only_channels
)

if __name__ == '__main__':
    decay_channels_utils.print_decay_channels_table(decay_channel_dict)
