"""
This file generates useful dictionaries for the low level cut study.

Patrick Ecker 2020
"""

from rdstar.studies.low_level_cut_study.utilities.final_state_particles_cut_study import \
    fsp_collection as fsp_collection_loose
from rdstar.studies.low_level_cut_study.utilities.final_state_particles_cut_study import \
    fsp_cut_sets as fsp_cut_sets_loose
from rdstar.online_analysis.final_state_particles import fsp_collection
from rdstar.online_analysis.final_state_particles import fsp_cut_sets

from rdstar.studies.low_level_cut_study.utilities.decay_channels_cut_study import \
    D_decay_label_to_cut_info_list as D_loose_dict, \
    Dstar_decay_label_to_cut_info_list as Dstar_loose_dict
from rdstar.online_analysis.decay_channels import D_decay_label_to_cut_info_list, Dstar_decay_label_to_cut_info_list, \
    D_decay_label_to_decay_id, Dstar_decay_label_to_decay_id, Dstar_decay_channels, D_decay_channels

fsp_ntuple_labels_dict = {
    "e_ntuple": "e",
    "mu_ntuple": "mu",
    "piCharged_ntuple": "pi+",
    "piNeutral_from_list_ntuple": "pi0",
    "gamma_ntuple": "gamma",
    "K_ntuple": "K+",
    "K_S0_ntuple": "K_S0"
}

fsp_cut_dict = {}
for n_tuple in fsp_ntuple_labels_dict.keys():
    fsps = [fsp for fsp in fsp_collection.all_fsps if fsp.label == fsp_ntuple_labels_dict[n_tuple]]
    assert len(fsps) == 1
    fsp = fsps[0]
    cut_infos = fsp_cut_sets[fsp.label]

    fsps_loose = [fsp for fsp in fsp_collection_loose.all_fsps if fsp.label == fsp_ntuple_labels_dict[n_tuple]]
    assert len(fsps_loose) == 1
    fsp = fsps_loose[0]
    cut_infos_loose = fsp_cut_sets_loose[fsp.label]

    fsp_cut_dict.update(
        {
            n_tuple: {
                "cuts": cut_infos,
                "cuts_loose": cut_infos_loose,
                "channel": ""
            }
        }
    )

ntuple_d_dict = {
    "dzero_ntuple": {
        "cuts": D_decay_label_to_cut_info_list,
        "cuts_loose": D_loose_dict,
        "label": "D0"
    },
    "dcharged_ntuple": {
        "cuts": D_decay_label_to_cut_info_list,
        "cuts_loose": D_loose_dict,
        "label": "D+"
    },
    "dstzero_ntuple": {
        "cuts": Dstar_decay_label_to_cut_info_list,
        "cuts_loose": Dstar_loose_dict,
        "label": "D*0"
    },
    "dstcharged_ntuple": {
        "cuts": Dstar_decay_label_to_cut_info_list,
        "cuts_loose": Dstar_loose_dict,
        "label": "D*+"
    }
}

D_cut_dict = {}
for n_tuple, cut_lists in ntuple_d_dict.items():
    assert cut_lists["cuts"].keys() == cut_lists["cuts_loose"].keys()
    for channel in cut_lists["cuts"].keys():
        if channel.startswith(cut_lists["label"]):
            D_cut_dict.update(
                {
                    f"{n_tuple}{channel}": {
                        "cuts": cut_lists["cuts"][channel],
                        "cuts_loose": cut_lists["cuts_loose"][channel],
                        "channel": channel
                    }
                }
            )

combined_decay_mode_id_dict = {}
combined_decay_mode_id_dict.update(D_decay_label_to_decay_id)
combined_decay_mode_id_dict.update(Dstar_decay_label_to_decay_id)

combined_channel_dict = {}
combined_channel_dict.update(D_decay_channels)
combined_channel_dict.update(Dstar_decay_channels)
combined_channel_dict.update({"": ""})

if __name__ == '__main__':
    for channel, item in Dstar_decay_label_to_cut_info_list.items():
        for c in item:
            print(c)

    for tuple, info in D_cut_dict.items():
        print(tuple)
        for item in info["cuts"]:
            print("cuts")
            print(item)
        for item in info["cuts_loose"]:
            print(item)

    print(fsp_cut_dict)
    print(D_cut_dict)
    print(combined_decay_mode_id_dict)
