import unittest
import numpy as np
import pandas as pd

from rdstar.mc_reweighting import NdimReweighter


class TestNdimReweighter(unittest.TestCase):

    def setUp(self):
        self.variables = ['x', 'y', 'z']
        self.bins = [3, 3, 3]
        self.old_model = pd.DataFrame(np.random.randn(10000, 3),
                                      columns=self.variables)
        self.new_model = pd.DataFrame(np.random.randn(10000, 3)*1.2,
                                      columns=self.variables)

    def test_coordinates(self):
        rw = NdimReweighter(self.old_model, self.new_model,
                            self.variables, self.bins)

        x = rw._bin_centers[0]
        expectedx1 = np.full(9, x[0])
        expectedx2 = np.full(9, x[1])
        expectedx3 = np.full(9, x[2])
        expectedx = np.append(expectedx1, (expectedx2, expectedx3))

        y = rw._bin_centers[1]
        expectedy = np.array([y[0], y[0], y[0], y[1], y[1], y[1], y[2], y[2], y[2]])

        z = rw._bin_centers[2]
        expectedz = np.array([z[0], z[1], z[2], z[0], z[1], z[2], z[0], z[1], z[2]])

        np.testing.assert_equal(expectedx, rw._coordinates_flattened[0])
        np.testing.assert_equal(np.append(np.array([]),
                                          (expectedy, expectedy, expectedy)),
                                rw._coordinates_flattened[1])
        np.testing.assert_equal(np.append(np.array([]),
                                          (expectedz, expectedz, expectedz)),
                                rw._coordinates_flattened[2])